//+----------------------------------------------------------------------------+
//|                                                               C_Orders.mqh |
//|                                                Copyright 2010, JQS aka Joo |
//|                                                        http://www.mql5.com |
//+----------------------------------------------------------------------------+
#property copyright "Copyright 2010, JQS aka Joo"
#property link      "http://www.mql5.com"
//
/*
  OP_BUY       0   
  OP_SELL      1   
  OP_BUYLIMIT  2    BUY LIMIT
  OP_SELLLIMIT 3    SELL LIMI
  OP_BUYSTOP   4    BUY STOP
  OP_SELLSTOP  5    SELL STOP
*/
//
class C_Orders
{
public:
  //-------------------- ---------------------------------------
  int    OrderType       []; //  
  double OrderVolume     []; //  
  int    OrderTimeOpen   []; //  
  double OrderPriceOpen  []; //  
  int    OrderTimeClose  []; //  
  double OrderPriceClose []; //  
  double OrderPriceSL    []; //  SL
  double OrderPriceTP    []; //  TP
  //double            OrderMaxDrawdown[]; //   
  double PipsResult      [];
  //----------------------------------------------------------------------------


  double Balance         []; //       0.
  int    TotalOrders;        //     
  int    MaxCountOrd;        //      
  //--------------------------
  void   Initialise       (int MaxPossibleCountOrd, double Spread, double Point_);
  int    OrderOpen        (int Type, double Volume, int Time,double Price,double SL, double TP);
  void   Possible_SL_or_TP(int Time, double PriceHigh,double PriceLow);
  void   OrderClose       (int Ticket, int Time,double Price);
  void   OrderCloseAll    (int Time, double   Price);
  int    ProfitTradeCount ();
  int    TotalPipsProfit  ();
  int    LossSeriesCount  ();
  int    ProfitSeriesCount();

private:
  double ask(double price);
  double bid(double price);
  double spred_point; //   0.00030
  double point;       //    0.00001
};
//
//
//    
void C_Orders::Initialise(int MaxPossibleCountOrd, double Spread, double Point_)
{
  point=Point_;
  spred_point=Spread*point;

  ArrayResize    (OrderType,       MaxPossibleCountOrd);
  ArrayInitialize(OrderType,       -1);

  ArrayResize    (OrderVolume,     MaxPossibleCountOrd);
  ArrayInitialize(OrderVolume,     0.0);

  ArrayResize    (OrderTimeOpen,   MaxPossibleCountOrd);
  ArrayInitialize(OrderTimeOpen,   1);
  ArrayResize    (OrderPriceOpen,  MaxPossibleCountOrd);
  ArrayInitialize(OrderPriceOpen,  0.0);

  ArrayResize    (OrderTimeClose,  MaxPossibleCountOrd);
  ArrayInitialize(OrderTimeClose,  1);
  ArrayResize    (OrderPriceClose, MaxPossibleCountOrd);
  ArrayInitialize(OrderPriceClose, 0.0);

  ArrayResize    (OrderPriceSL,    MaxPossibleCountOrd);
  ArrayInitialize(OrderPriceSL,    0.0);
  ArrayResize    (OrderPriceTP,    MaxPossibleCountOrd);
  ArrayInitialize(OrderPriceTP,    0.0);

  //ArrayResize    (OrderMaxDrawdown,MaxPossibleCountOrd);
  //ArrayInitialize(OrderMaxDrawdown,0);

  ArrayResize    (PipsResult,      MaxPossibleCountOrd);
  ArrayInitialize(PipsResult,      0.0);

  ArrayResize    (Balance,         MaxPossibleCountOrd+1);
  ArrayInitialize(Balance,         0.0);

  TotalOrders=0;
  MaxCountOrd=MaxPossibleCountOrd;
}
//

//
//   
double C_Orders::ask(double price){return(price+spred_point);}
//

//
//   
double C_Orders::bid(double price){return(price);}
//

//
//  .    
//...(       0),
//...    -1
//
int C_Orders::OrderOpen(int    Type,
                        double Volume,
                        int    Time,
                        double Price,
                        double SL,     //  
                        double TP      //  
                       )
{
  //           -1     0
  if (TotalOrders<MaxCountOrd && (Type==0 || Type==1) && Volume>0.0)
  {
    //-------------------------------------
    //    buy
    if (Type==0)
    {
      OrderPriceOpen[TotalOrders]=ask(Price);
      if (TP>0.0)
        OrderPriceTP[TotalOrders]=ask(Price)+TP*point;
      else
        OrderPriceTP[TotalOrders]=0.0;
      if (SL>0)
        OrderPriceSL[TotalOrders]=ask(Price)-SL*point;
      else
        OrderPriceSL[TotalOrders]=0.0;
    }

    //    sell
    if (Type==1)
    {
      OrderPriceOpen[TotalOrders]=bid(Price);
      if (TP>0.0)
        OrderPriceTP[TotalOrders]=bid(Price)-TP*point;
      else
        OrderPriceTP[TotalOrders]=0.0;
      if (SL>0)
        OrderPriceSL[TotalOrders]=bid(Price)+SL*point;
      else
        OrderPriceSL[TotalOrders]=0.0;
    }

    //-------------------------------------
    //  
    OrderType[TotalOrders]=Type;
    //-------------------------------------
    //  
    OrderVolume[TotalOrders]=Volume;
    //-------------------------------------
    //  
    OrderTimeOpen[TotalOrders]=Time;

    TotalOrders++;

    return(TotalOrders-1);
  }
  else
    return(-1);
}
//

//
//     
void C_Orders::OrderClose(int Ticket, int Time, double Price)
{
  //             )
  if (Ticket<=TotalOrders-1 && OrderPriceOpen[Ticket]!=0.0)
    //      (   0)
    if (OrderPriceClose[Ticket]==0.0)
    {
      OrderTimeClose [Ticket]=Time;
      //------------------------------------------------------------------------
      //   buy
      if (OrderType[Ticket]==0)
      {
        OrderPriceClose[Ticket]=bid(Price);
        PipsResult[Ticket]=OrderPriceClose[Ticket]-OrderPriceOpen[Ticket];
      }
      //------------------------------------------------------------------------
      //   sell
      if (OrderType[Ticket]==1)
      {
        OrderPriceClose[Ticket]=ask(Price);
        PipsResult[Ticket]=OrderPriceOpen[Ticket]-OrderPriceClose[Ticket];
      }
    }
}
//

//
//    
void C_Orders::OrderCloseAll(int Time,double Price)
{
  //   
  for (int i=0; i<TotalOrders;i++)
    //      (   0)
    if (OrderPriceClose[i]==0.0)
    {
      OrderTimeClose [i]=Time;
      //------------------------------------------------------------------------
      //   buy
      if (OrderType[i]==0)
      {
        OrderPriceClose[i]=bid(Price);
        PipsResult[i]=OrderPriceClose[i]-OrderPriceOpen[i];
      }
      //------------------------------------------------------------------------
      //   sell
      if (OrderType[i]==1)
      {
        OrderPriceClose[i]=ask(Price);
        PipsResult[i]=OrderPriceOpen[i]-OrderPriceClose[i];
      }
    }
}
//

//
//    SL  TP   
void C_Orders::Possible_SL_or_TP(int Time,double PriceHigh,double PriceLow)
{
  //   
  for (int i=0; i<TotalOrders;i++)
  {
    //    
    if (OrderPriceClose[i]==0.0)
    {
      //------------------------------------------------------------------------
      //   buy
      if (OrderType[i]==0)
      {
        //   SL
        if (OrderPriceSL[i]>=bid(PriceLow))
        {
          OrderPriceClose[i]=OrderPriceSL[i];
          OrderTimeClose[i]=Time;
          PipsResult[i]=OrderPriceClose[i]-OrderPriceOpen[i];
        }
        else
        {
          // ...    TP
          if (OrderPriceTP[i]<bid(PriceHigh))
          {
            OrderPriceClose[i]=OrderPriceTP[i];
            OrderTimeClose[i]=Time;
            PipsResult[i]=OrderPriceClose[i]-OrderPriceOpen[i];
          }
        }
      }
      //------------------------------------------------------------------------
      //   sell
      if (OrderType[i]==1)
      {
        //   SL
        if (OrderPriceSL[i]<=ask(PriceHigh))
        {
          OrderPriceClose[i]=OrderPriceSL[i];
          OrderTimeClose [i]=Time;
          PipsResult[i]=OrderPriceOpen[i]-OrderPriceClose[i];
        }
        else
        {
          // ...    TP
          if (OrderPriceTP[i]>ask(PriceLow))
          {
            OrderPriceClose[i]=OrderPriceTP[i];
            OrderTimeClose[i]=Time;
            PipsResult[i]=OrderPriceOpen[i]-OrderPriceClose[i];
          }
        }
      }
      //------------------------------------------------------------------------
    }
  }
}
//

//
//    
int C_Orders::ProfitTradeCount()
{
  int profitTradeCount=0; //   
  //      
  for (int i=0; i<TotalOrders;i++)
    //   
    if (OrderPriceClose[i]!=0.0)
      //    
      if (PipsResult[i]>0.0)
        //  1    
        profitTradeCount++;

  return(profitTradeCount);
}
//

//
//           
//...   0- 
int C_Orders::TotalPipsProfit()
{
  for (int i=0; i<TotalOrders;i++)
    //   
    if (OrderPriceClose[i]!=0.0)
      Balance[i+1]=Balance[i]+PipsResult[i]*OrderVolume[i];
    Print(point);
  return(int(Balance[TotalOrders]/point));
}
//

//
//       
int C_Orders::LossSeriesCount()
{
  int cntMaxLossSeries=1;
  int cntLossSeries   =0;

  for (int i=0; i<TotalOrders;i++)
  {
    //   
    if (OrderPriceClose[i]!=0.0)
    {
      if (PipsResult[i]<=0.0)
      {
        cntLossSeries++;
        if (cntLossSeries>cntMaxLossSeries)
          cntMaxLossSeries=cntLossSeries;
      }
      else
        cntLossSeries=0;
    }
  }
  return(cntMaxLossSeries);
}
//

//
//       
int C_Orders::ProfitSeriesCount()
{
  int cntMaxProfitSeries=1;
  int cntProfitSeries   =0;

  for (int i=0; i<TotalOrders;i++)
  {
    //   
    if (OrderPriceClose[i]!=0.0)
    {
      if (PipsResult[i]>0.0)
      {
        cntProfitSeries++;
        if (cntProfitSeries>cntMaxProfitSeries)
          cntMaxProfitSeries=cntProfitSeries;
      }
      else
        cntProfitSeries=0;
    }
  }
  return(cntMaxProfitSeries);
}
//
/*
//
void C_Orders::OrdersDraw()
{
  //   
  for (int i=0; i<TotalOrders;i++)
  {
    ObjectCreate(
                0,                  //  
                (string)i,          //  
                OBJ_TREND,          //  
                0,                  //  
                OrderTimeOpen  [i], //    
                OrderPriceOpen [i], //    

                OrderTimeClose [i], //  N-  
                OrderPriceClose[i]  //  N-  
                );
    if (OrderVolume[i]<0.0)
    {
      if (OrderPriceOpen[i]>OrderPriceClose[i])
        ObjectSetInteger(0,(string)i,OBJPROP_COLOR,clrBlue);
      else
        ObjectSetInteger(0,(string)i,OBJPROP_COLOR,clrRed);
    }

    if (OrderVolume[i]>0.0)
    {
      if (OrderPriceOpen[i]<OrderPriceClose[i])
        ObjectSetInteger(0,(string)i,OBJPROP_COLOR,clrBlue);
      else
        ObjectSetInteger(0,(string)i,OBJPROP_COLOR,clrRed);
    }

    ObjectSetInteger(0,(string)i,OBJPROP_SELECTABLE,true);
    ObjectSetInteger(0,(string)i,OBJPROP_WIDTH,3);
  }
}
//
*/
